; $Id: flash.asm 179 2008-03-08 12:09:54Z tk $
;
;  Flash Access Routines
;
; ==========================================================================
;
;  Copyright 2006 Thorsten Klose (tk@midibox.org)
;  Licensed for personal non-commercial use only.
;  All other rights reserved.
; 
; ==========================================================================

	list	p=16f88
	radix	dec
#include <p16f88.inc>
#include "macros.inc"
#include "globals.inc"

; ==========================================================================

	;; export lables
	global	FLASH_ReadInc

; ==========================================================================
FLASH code
; ==========================================================================

;; --------------------------------------------------------------------------
;;  FUNCTION: FLASH_ReadInc
;;  DESCRIPTION: Read data from FLASH and increments address pointer
;;  IN:	  address in EEADR[H]
;;  OUT:  8bit LSB in EEDATA and WREG, 6bit MSB in EEDATH
;;  USES: -
;; --------------------------------------------------------------------------
FLASH_ReadInc
	SWITCHBANK_0_3
	bsf	EECON1, EEPGD		; point to program memory
	bsf	EECON1, RD		; FLASH Read
	nop
	nop
	SWITCHBANK_3_2

	incf	EEADR, F
        skpnz
        incf    EEADRH, F

	movf	EEDATA, W
	SWITCHBANK_2_0
	return

;; --------------------------------------------------------------------------
;;  FUNCTION: EEPROM_Write
;;  DESCRIPTION: Write data to FLASH and increments address pointer
;;  IN: address in EEADR[H], Data byte in WREG
;;  OUT:  
;;  USES: -
;; --------------------------------------------------------------------------
EEPROM_Write
	SWITCHBANK_0_3
	btfsc	EECON1,WR ; wait for existing write
	goto	$-1
	movwf EEDATA
	bcf EECON1, EEPGD
	bsf EECON1, WREN
	IRQ_DISABLE
	movwf EECON2
	movlw 0x55
	movwf EECON2
	movlw 0xaa
	movwf EECON2
	bsf EECON1,WR
	IRQ_ENABLE
	bcf EECON1,WREN
	SWITCHBANK_3_2
	incf	EEADR, F
	SWITCHBANK_2_0
	return
	
;; --------------------------------------------------------------------------
;;  FUNCTION: EEPROM_Read
;;  DESCRIPTION: Read data from EEPROM and increments address pointer
;;  IN:	  address in EEADR[H]
;;  OUT:  8bit byte in EEDATA and WREG,
;;  USES: -
;; --------------------------------------------------------------------------
EEPROM_Read
	SWITCHBANK_0_3
	bcf	EECON1, EEPGD		; point to data memory
	bsf	EECON1, RD		; EEPROM Read
	SWITCHBANK_3_2
	incf	EEADR, F
	movf	EEDATA, W
	SWITCHBANK_2_0
	return

; ==========================================================================
	END
