; $Id: iic.asm 178 2008-10-10 12:08:54Z pt $
;
;  IIC (Slave) Driver
;
; This has only been slightly modified to speak DMX rather than MIDI
; I have left in the IIC_Handler_Read as I hope to have some DMX In functionality 
; There is loads of program space free so no problem!
;
; ==========================================================================
;
;  Copyright 2008 Phil Taylor (phil@taylor.org.uk)
;  Copyright 2006 Thorsten Klose (tk@midibox.org)
;  Licensed for personal non-commercial use only.
;  All other rights reserved.
; 
; ==========================================================================
;  device address of IIC slave (the ID selected at RB[6:7] is ored to [2:1] of the address
;  Note: MUST be an even number, because bit #0 indicates a read/write operation
; ==========================================================================
IIC_DEVICE_ADDRESS	EQU	0x10
;
; ==========================================================================
;
;  Copyright 2006 Thorsten Klose (tk@midibox.org)
;  Licensed for personal non-commercial use only.
;  All other rights reserved.
; 
; ==========================================================================

	list	p=16f88
	radix	dec
#include <p16f88.inc>
#include "macros.inc"
#include "globals.inc"

; ==========================================================================

	;; export lables
	global	IIC_Init
	global	IIC_Init_GetAddress
	global	IIC_Handler
	global	IIC_RxBufferUsed
	global	IIC_RxBufferGet
	
	extern	DMX_RxBufferPut

	;extern	DMX_TxBufferPut
	;extern	DMX_TxBufferFree

	extern	DMX_LEDS_Handler
	extern	DMX_Get_Location
	extern	DMX_BANK1
	extern	DMX_BANK2
	extern	DMX_BANK3
	extern	DMX_BANK4
	extern  DMX_RX_SIZE
	extern	DMX_PACKAGE
	global  IIC_DATA ; Use for TX/RX IIC
	; These 3 are used to stream all channel values back to the core on request.
	extern	DMX_COMMAND

; ==========================================================================
gpr0				udata
#define IIC_RX_BUFFER_SIZE 0x10
IIC_RX_BUFFER		res IIC_RX_BUFFER_SIZE
IIC_RX_BUFFER_TMP	res	1
IIC_RX_BUFFER_HEAD	res	1
IIC_RX_BUFFER_TAIL	res	1
IIC_RX_BUFFER_CTR	res	1

#define IIC_TX_BUFFER_SIZE 0x10
IIC_TX_BUFFER		res IIC_TX_BUFFER_SIZE
IIC_TX_BUFFER_TMP	res	1
IIC_TX_BUFFER_HEAD	res	1
IIC_TX_BUFFER_TAIL	res	1
IIC_TX_BUFFER_CTR	res	1

shared_ram			udata_shr
IIC_TMP				res	1
IIC_DATA			res	1
IIC_READ_CTR		res	1
; ==========================================================================
IIC	code
; ==========================================================================

;; --------------------------------------------------------------------------
;;  FUNCTION: IIC_Init
;;  DESCRIPTION: initialises the IIC interface
;;  IN:	  -
;;  OUT:  -
;;  USES: -
;; --------------------------------------------------------------------------
IIC_Init
	SWITCHBANK_0_1
	bsf PCON, NOT_POR
	bsf PCON, NOT_BOR
	SWITCHBANK_1_0

	;; setup SSP module for 7-bit
	movlw	0x36
	movwf	SSPCON

	;; determine address (depending on IDx pins)
	call	IIC_Init_GetAddress

	SWITCHBANK_0_1
	clrf	SSPSTAT
	SWITCHBANK_1_0
	; Clear the receive interrupt pin (this will get set when we want to send some data)
	bsf	RI_N_PORT, RI_N_PIN	; set RI_N=1 (receive interrupt) pin - it's low active!
	clrf IIC_READ_CTR ; Used to send DMX dump via IIC (I can't use the other one as we will probably not be aligned!
	
	; initialize ringbuffers
	clrf	IIC_RX_BUFFER_TAIL
	clrf	IIC_RX_BUFFER_HEAD
	clrf	IIC_RX_BUFFER_CTR

	clrf	IIC_TX_BUFFER_TAIL
	clrf	IIC_TX_BUFFER_HEAD
	clrf	IIC_TX_BUFFER_CTR
	return

;; --------------------------------------------------------------------------
;;  FUNCTION: IIC_Init_GetAddress
;;  DESCRIPTION: initialises the IIC address only - can be called from the
;;  mainloop to allow the change on-the-fly without device reset
;;  IN:	  -
;;  OUT:  -
;;  USES: -
;; --------------------------------------------------------------------------
IIC_Init_GetAddress
	movlw	IIC_DEVICE_ADDRESS

	btfsc	ID0_PORT, ID0_PIN
	iorlw	0x02
	btfsc	ID1_PORT, ID1_PIN
	iorlw	0x04

	SWITCHBANK_0_1
	movwf	SSPADD
	SWITCHBANK_1_0
	return

	
	
IIC_RxBufferUsed
	movf	IIC_RX_BUFFER_CTR, W
	return	

IIC_RxBufferFree
	movf	IIC_RX_BUFFER_CTR, W
	sublw	IIC_RX_BUFFER_SIZE-1
	return
	
IIC_RxBufferPut
	movwf	IIC_RX_BUFFER_TMP		; save byte which should be sent

	incf	IIC_RX_BUFFER_TAIL, W		; increment tail ptr, wrap if end of buffer reached
	xorlw	IIC_RX_BUFFER_SIZE
	skpz
	xorlw	IIC_RX_BUFFER_SIZE		; XOR back to old value
	movwf	FSR
	xorwf	IIC_RX_BUFFER_HEAD, W
	skpnz
	return					; avoid overrun!

	movf	FSR, W				; prepare buffer pointer
	movwf	IIC_RX_BUFFER_TAIL
	addlw	LOW(IIC_RX_BUFFER)
	movwf	FSR
	movf	IIC_RX_BUFFER_TMP, W		; push byte onto buffer
	movwf	INDF
	incf	IIC_RX_BUFFER_CTR, F
	return

IIC_RxBufferGet
	incf	IIC_RX_BUFFER_HEAD, F
	movf	IIC_RX_BUFFER_HEAD, W
	xorlw	IIC_RX_BUFFER_SIZE
	skpnz
	clrf	IIC_RX_BUFFER_HEAD

	movf	IIC_RX_BUFFER_HEAD, W
	addlw	LOW(IIC_RX_BUFFER)
	movwf	FSR
	movf	INDF, W
	decf	IIC_RX_BUFFER_CTR, F
	return

IIC_TxBufferUsed
	movf	IIC_TX_BUFFER_CTR, W
	return

IIC_TxBufferFree
	movf	IIC_TX_BUFFER_CTR, W
	sublw	IIC_TX_BUFFER_SIZE-1
	return


	
	
IIC_TxBufferPut
	movwf	IIC_TX_BUFFER_TMP		; save byte which should be sent

	incf	IIC_TX_BUFFER_TAIL, W		; increment tail ptr, wrap if end of buffer reached
	xorlw	IIC_TX_BUFFER_SIZE
	skpz
	xorlw	IIC_TX_BUFFER_SIZE		; XOR back to old value
	movwf	FSR
	xorwf	IIC_TX_BUFFER_HEAD, W
	bnz	IIC_TxBufferPut_Ok

	;; repeat until buffer is free again
	IRQ_ENABLE		; (we must ensure that interrupts are enabled!)
	movf	IIC_TX_BUFFER_TMP, W
	goto	IIC_TxBufferPut

IIC_TxBufferPut_Ok
	IRQ_DISABLE				; disable interrupts to avoid a malfunction when a Tx IRQ happens here
	movf	FSR, W				; prepare buffer pointer
	movwf	IIC_TX_BUFFER_TAIL
	addlw	LOW(IIC_TX_BUFFER)
	movwf	FSR
	movf	IIC_TX_BUFFER_TMP, W		; push byte onto buffer
	movwf	INDF
	incf	IIC_TX_BUFFER_CTR, F

	SWITCHBANK_0_1
	bsf	PIE1, TXIE			; (re-)enable transmit IRQ
	SWITCHBANK_1_0

	IRQ_ENABLE				; enable interrupts again

	return

	
IIC_TxBufferGet
	incf	IIC_TX_BUFFER_HEAD, F
	movf	IIC_TX_BUFFER_HEAD, W
	xorlw	IIC_TX_BUFFER_SIZE
	skpnz
	clrf	IIC_TX_BUFFER_HEAD

	movf	IIC_TX_BUFFER_HEAD, W
	addlw	LOW(IIC_TX_BUFFER)
	movwf	FSR
	movf	INDF, W
	decf	IIC_TX_BUFFER_CTR, F
	return
	
IIC_TxBufferFlush
	clrwdt
IIC_TxBufferFlush_WaitTx
	call	IIC_TxBufferUsed
	bnz	IIC_TxBufferFlush_WaitTx
	return

;; --------------------------------------------------------------------------
;;  FUNCTION: IIC_Handler
;;  DESCRIPTION: handles IIC transactions, should be called from the mainloop
;;               on IIC events.
;;  IN:	  -
;;  OUT:  -
;;  USES: -
;; --------------------------------------------------------------------------
IIC_Handler
	;; get value of SSPSTAT and store it in IIC_TMP
	SWITCHBANK_0_1
	movf	SSPSTAT, W
	SWITCHBANK_1_0
	movwf	IIC_TMP

	;; note: in distance to AN734 we don't check the start bit here,
	;; because there are cases where the last byte has been received
	;; and the stop bit is already set

	;; branch depending on BF flag (BF=1 -> write, otherwise read operation)
	btfss	IIC_TMP, BF
	goto	IIC_Handler_Read

IIC_Handler_Write

	;; check for SSPOV flag - if overrun happens, wait for stop flag, thereafter clear SSPOV flag
	btfss	SSPCON, SSPOV
	goto	IIC_Handler_Write_NoOV
	SWITCHBANK_0_1
IIC_Handler_Write_OVLoop
	btfss	SSPSTAT, P
	goto	IIC_Handler_Write_OVLoop
	SWITCHBANK_1_0
	bcf	SSPCON, SSPOV		; first clear SSPOV, then BF bit
	movf	SSPBUF, W
	goto	IIC_Handler_End

IIC_Handler_Write_NoOV

	movf	SSPBUF, W		; Get the byte from the SSP (very early!)
	movwf	IIC_DATA		; and save it in IIC_DATA

	;; branch depending on D_A flag (D_A=0 -> address, otherwise data byte)
	btfsc	IIC_TMP, D_A
	goto	IIC_Handler_WriteB

IIC_Handler_WriteA	; (write phase, address)
	;; nothing more to do
	goto	IIC_Handler_Write_End

IIC_Handler_WriteB	; (write phase, data)
	call	IIC_RxBufferFree
	bz		IIC_Handler_Write_Error ; Buffer is full
	movf	IIC_DATA, W
	call	IIC_RxBufferPut
	;call DMX_ChannelSet ; Write the DMX channel data to memory 
	clrf IIC_DATA
IIC_Handler_WriteB_End
	;; check for overrun - if this happens, wait for stop bit and clear the SSPOV flag
	btfss	SSPCON, SSPOV
	goto	IIC_Handler_Write_End	; exit handler

	SWITCHBANK_0_1
IIC_Handler_WriteB_OVLoop
	btfss	SSPSTAT, P
	goto	IIC_Handler_WriteB_OVLoop
	SWITCHBANK_1_0

	;; first clear SSPOV, then BF bit
	bcf	SSPCON, SSPOV
	movf	SSPBUF, W
	goto	IIC_Handler_End

IIC_Handler_Write_End
	;; fast path if SSPIF and BF flag set
	btfss	PIR1, SSPIF
	goto	IIC_Handler_End	; exit handler
	SWITCHBANK_0_1
	movf	SSPSTAT, W
	SWITCHBANK_1_0
	movwf	IIC_TMP
	btfss	IIC_TMP, BF
	goto	IIC_Handler_End	; exit handler

	bcf	PIR1, SSPIF	; clear SSP flag
	goto	IIC_Handler_Write

IIC_Handler_Write_Error ; Gets called on buffer overflow
	#if LED_PWR_PORT
		bcf	LED_PWR_PORT, LED_PWR_PIN
	#endif
	goto IIC_Handler_End
	
IIC_Handler_Read
	;; branch depending on D_A flag (D_A=0 -> address, otherwise data byte)
	btfsc	IIC_TMP, D_A
	goto	IIC_Handler_ReadB

IIC_Handler_ReadA	; (read phase, address)
	;; clear read counter and return first byte of package
	clrf	IIC_READ_CTR
	movf	DMX_PACKAGE+0, W
	call	IIC_Write
	goto	IIC_Handler_Read_End	; exit read handler

IIC_Handler_ReadB	; (write phase, data)
	;; branch depending on R_W flag (R_W = 1 -> data byte should be read, otherwise NAK has been received)
	btfss	IIC_TMP, R_W
	goto	IIC_Handler_ReadNAK
	
IIC_Handler_ReadACK

	bsf	RI_N_PORT, RI_N_PIN	; set RI_N=1 (receive interrupt) pin - it's low active!
					; the pin was set to 0 in dmx.asm

	;; increment read counter and return value of package
	incf	IIC_READ_CTR, F
	movlw	DMX_PACKAGE
	addwf	IIC_READ_CTR, W
	movwf	FSR
	movf	INDF, W
	call	IIC_Write

	;; clear package once we reached CTR >= 3
	incf	IIC_READ_CTR, W
	andlw	0xfc
	bz	IIC_Handler_ReadACK_NoClr
IIC_Handler_ReadACK_Clr
	clrf	IIC_READ_CTR
	
IIC_Handler_ReadACK_NoClr

	goto	IIC_Handler_Read_End	; exit read handler

IIC_Handler_ReadNAK
	bsf	SSPCON, CKP		; release clock
	;; 	goto	IIC_Handler_Read_End	; exit read handler

IIC_Handler_Read_End

IIC_Handler_End
	return



;; --------------------------------------------------------------------------
;;  FUNCTION: IIC_Write
;;  DESCRIPTION: sends a byte over IIC bus
;;  IN:	  byte to send in WREG
;;  OUT:  -
;;  USES: -
;; --------------------------------------------------------------------------
IIC_Write
	SWITCHBANK_0_1
IIC_Write_Wait
	btfsc	SSPSTAT, BF		; wait until buffer empty
	goto	IIC_Write_Wait
	SWITCHBANK_1_0
IIC_Write_Now
	bcf	SSPCON, WCOL		; Clear the WCOL flag.
	movwf	SSPBUF			; Write the byte in WREG
	btfss	SSPCON, WCOL		; write again on collision
	goto	IIC_Write_Now
	bsf	SSPCON, CKP		; release clock
	return

; ==========================================================================
	END
