; $Id: iic_cmd.asm 178 2008-03-08 12:08:54Z tk $
;
;  IIC Command Handler
;
;  Called from IIC_Handler if command mode has been selected with the 0xff token
;
;  Following sequences will *not* call the command handler:

;     - 0xff 0x00 (Exit Command Mode)
;     - 0xff 0xff (Send 0xff via MIDI and exit Command Mode)
;
;  All other sequences like:
;    0xff <cmd> <data1> <data2> <data3> ... <data_n>
;  Will call the command handler whenever a new data byte has been received
;  (command in IIC_CMD, received data byte in WREG)
;  
;  Command mode will be exit once the command handler returns 0x00
;
; ==========================================================================
;
;  Copyright 2006 Thorsten Klose (tk@midibox.org)
;  Licensed for personal non-commercial use only.
;  All other rights reserved.
; 
; ==========================================================================

	list	p=16f88
	radix	dec
#include <p16f88.inc>
#include "macros.inc"
#include "globals.inc"


; variables used by BSL access the memory directly!
; We have all memory available (MUST modify if ANY of these change)


DEVICE_ID       	EQU  	0x21
CHECKSUM        	EQU  	0x22
TMP1			EQU  	0x23
TMP2			EQU  	0x24
TMP3			EQU  	0x25

BYTE_NUMBER     	EQU  	0x26
RECEIVED_BYTE   	EQU  	0x27

SYX_U			EQU  	0x28
SYX_H			EQU  	0x29
SYX_L			EQU  	0x2a

SYX_BIT7_CTR		EQU  	0x2b
SYX_BIT8_CTR		EQU  	0x2c
SYX_BYTE7       	EQU  	0x2d
SYX_BYTE8       	EQU  	0x2e

BYTE_CTR_H      	EQU  	0x30
BYTE_CTR_L      	EQU  	0x31

FLASH_BLOCK_CTR		EQU  	0x32
FLASH_ROW_CTR		EQU  	0x33
FLASH_BUFFER_CTR 	EQU 	0x34

TIMEOUT_CTR_U		EQU  	0x40
TIMEOUT_CTR_H		EQU  	0x41
TIMEOUT_CTR_L		EQU  	0x42

ADDRESS_EXTENSION	EQU	0x43

UART_TMP		EQU	0x44

IIC_MIDI_ADDR		EQU	0x45
IIC_MIDI_TMP		EQU	0x46
IIC_MIDI_RETRY_CTR	EQU	0x47

IIC_STAT		EQU	0x48
IIC_CTR			EQU	0x49
IIC_BUFFER		EQU	0x4a

MIDI_BUFFER     	EQU  	0xa0
MIDI_BUFFER_END 	EQU  	0xef



; ==========================================================================

	;; export lables
	global	IIC_CMD_Handler

	;; import lables
	extern	DMX_BUF_POINTERA
	extern	DMX_BUF_POINTERB
	extern	DMX_BUFFER_TMP
	extern	DMX_RX_SIZE
	extern	DMX_TMP_COMMAND

	;extern	MIDI_TxBufferPut

; ==========================================================================

; ==========================================================================
IIC_CMD	code
; ==========================================================================




;; --------------------------------------------------------------------------
;;  FUNCTION: IIC_CMD_Handler
;;  DESCRIPTION: see header of this file
;;  IN:	  command in IIC_CMD, data in WREG
;;  OUT:  -
;;  USES: -
;; --------------------------------------------------------------------------
IIC_CMD_Handler
;; ignore realtime messages
	movlw	0xf8
	BRES RECEIVED_BYTE, IIC_CMD_Handler_End ; This byte is realtime so ignore.
	return
	
IIC_CMD_Handler_End
	return
	
	

; ==========================================================================
	END
